package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.FieldReference;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ProblemFieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.ProblemMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class JavadocFieldReference extends FieldReference {
  public int tagSourceStart;
  
  public int tagSourceEnd;
  
  public int tagValue;
  
  public MethodBinding methodBinding;
  
  public JavadocFieldReference(char[] source, long pos) {
    super(source, pos);
    this.bits |= 0x8000;
  }
  
  protected TypeBinding internalResolveType(Scope scope) {
    FieldBinding fieldBinding1;
    ProblemFieldBinding problemFieldBinding;
    this.constant = Constant.NotAConstant;
    if (this.receiver == null) {
      this.actualReceiverType = (TypeBinding)scope.enclosingReceiverType();
    } else if (scope.kind == 3) {
      this.actualReceiverType = this.receiver.resolveType((ClassScope)scope);
    } else {
      this.actualReceiverType = this.receiver.resolveType((BlockScope)scope);
    } 
    if (this.actualReceiverType == null)
      return null; 
    Binding fieldBinding = (this.receiver != null && this.receiver.isThis()) ? 
      scope.classScope().getBinding(this.token, this.bits & 0x7, (InvocationSite)this, true) : 
      (Binding)scope.getField(this.actualReceiverType, this.token, (InvocationSite)this);
    if (!fieldBinding.isValidBinding()) {
      FieldBinding closestMatch;
      switch (fieldBinding.problemId()) {
        case 5:
        case 6:
        case 7:
          closestMatch = ((ProblemFieldBinding)fieldBinding).closestMatch;
          if (closestMatch != null)
            fieldBinding1 = closestMatch; 
          break;
      } 
    } 
    if (!fieldBinding1.isValidBinding() || !(fieldBinding1 instanceof FieldBinding)) {
      if (this.receiver.resolvedType instanceof org.eclipse.jdt.internal.compiler.lookup.ProblemReferenceBinding)
        return null; 
      if (this.actualReceiverType instanceof ReferenceBinding) {
        ReferenceBinding refBinding = (ReferenceBinding)this.actualReceiverType;
        char[] selector = this.token;
        MethodBinding possibleMethod = null;
        if (CharOperation.equals(this.actualReceiverType.sourceName(), selector)) {
          possibleMethod = scope.getConstructor(refBinding, Binding.NO_TYPES, (InvocationSite)this);
        } else {
          possibleMethod = this.receiver.isThis() ? 
            scope.getImplicitMethod(selector, Binding.NO_TYPES, (InvocationSite)this) : 
            scope.getMethod((TypeBinding)refBinding, selector, Binding.NO_TYPES, (InvocationSite)this);
        } 
        if (possibleMethod.isValidBinding()) {
          this.methodBinding = possibleMethod;
        } else {
          ProblemMethodBinding problemMethodBinding = (ProblemMethodBinding)possibleMethod;
          if (problemMethodBinding.closestMatch == null) {
            if (fieldBinding1.isValidBinding())
              problemFieldBinding = new ProblemFieldBinding(refBinding, fieldBinding1.readableName(), 1); 
            scope.problemReporter().javadocInvalidField(this, (Binding)problemFieldBinding, this.actualReceiverType, scope.getDeclarationModifiers());
          } else {
            this.methodBinding = problemMethodBinding.closestMatch;
          } 
        } 
      } 
      return null;
    } 
    this.binding = (FieldBinding)problemFieldBinding;
    if (isFieldUseDeprecated(this.binding, scope, this.bits))
      scope.problemReporter().javadocDeprecatedField(this.binding, (ASTNode)this, scope.getDeclarationModifiers()); 
    return this.resolvedType = this.binding.type;
  }
  
  public boolean isSuperAccess() {
    return ((this.bits & 0x4000) != 0);
  }
  
  public StringBuffer printExpression(int indent, StringBuffer output) {
    if (this.receiver != null)
      this.receiver.printExpression(0, output); 
    output.append('#').append(this.token);
    return output;
  }
  
  public TypeBinding resolveType(BlockScope scope) {
    return internalResolveType((Scope)scope);
  }
  
  public TypeBinding resolveType(ClassScope scope) {
    return internalResolveType((Scope)scope);
  }
  
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    if (visitor.visit(this, scope) && 
      this.receiver != null)
      this.receiver.traverse(visitor, scope); 
    visitor.endVisit(this, scope);
  }
  
  public void traverse(ASTVisitor visitor, ClassScope scope) {
    if (visitor.visit(this, scope) && 
      this.receiver != null)
      this.receiver.traverse(visitor, scope); 
    visitor.endVisit(this, scope);
  }
}
